#!/usr/bin/env ruby

require 'open3'

failed_interpreter = []

['./waker.rb', './wake'].each do |interpreter|
  puts "=== Testing #{interpreter} ==="

  failed = []

  Dir.glob('test/*.wake').each do |test|
    test = test[/(.*)\.wake$/, 1]

    # Skip slow test
    if test =~ /bfint/
      next if interpreter =~ /\.rb$/
    end

    # Hmm... This test doesn't finish with make
    if ENV['FROM_MAKE'] && test =~ /stack_overflow/
      next
    end

    print "#{test}... "

    cmd = "#{interpreter} #{test}.wake"
    o, err = Open3.popen3(cmd) do |stdin, stdout, stderr|
      inf = "#{test}.in"
      if File.exist?(inf)
        stdin.print(File.read(inf))
      end
      stdin.close

      [stdout.read.strip, stderr.read.strip]
    end

    e = "#{test}.txt"
    if File.exist?(e)
      e = File.read(e).strip
      if e == o
        puts 'OK'
      else
        failed << test
        puts 'FAIL'
        puts "expected: #{e} actual: #{o}"
      end
    else
      failed << test
      puts "Missing expectation - actual: #{o}"
    end

    e = "#{test}.err"
    e = File.exist?(e) ? e = File.read(e).strip : ""
    if e[0] == ?!
      e = Regexp.new(e[1..-1])
    end
    if !(e === err)
      failed << test
      puts 'FAIL (stderr)'
      puts "expected: #{e} actual: #{err}"
    end
  end

  if failed.empty?
    puts 'ALL TESTS PASS'
  else
    failed_interpreter << [interpreter, failed]
    puts "FAILED: #{failed * ' '}"
  end
  puts
end

if failed_interpreter.empty?
  puts 'ALL INTERPRETERS ARE SANE'
else
  puts 'SOME INTERPRETERS FAIL'
  failed_interpreter.each do |interpreter, failed|
    puts "#{interpreter}: #{failed * ' '}"
  end
end
